/*
 * Sixth 3D engine. Copyright ©2012-2018, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.geometry;

import static java.lang.Math.sqrt;

public class Point2D implements Cloneable {

    public double x, y;

    public Point2D() {
    }

    public Point2D(final double x, final double y) {
        this.x = x;
        this.y = y;
    }

    public Point2D(final Point2D source) {
        x = source.x;
        y = source.y;
    }

    public Point2D add(final Point2D direction) {
        x += direction.x;
        y += direction.y;
        return this;
    }

    public boolean isZero() {
        return (x == 0) && (y == 0);
    }

    @Override
    public Point2D clone() {
        return new Point2D(this);
    }

    public void clone(final Point2D source) {
        x = source.x;
        y = source.y;
    }

    public Point2D getMiddle(final Point2D p1, final Point2D p2) {
        x = (p1.x + p2.x) / 2d;
        y = (p1.y + p2.y) / 2d;
        return this;
    }

    public double getAngleXY(final Point2D anotherPoint) {
        return Math.atan2(x - anotherPoint.x, y - anotherPoint.y);
    }

    /**
     * Compute distance to another point.
     */
    public double getDistanceTo(final Point2D anotherPoint) {
        final double xDiff = x - anotherPoint.x;
        final double yDiff = y - anotherPoint.y;

        return sqrt(((xDiff * xDiff) + (yDiff * yDiff)));
    }

    public double getVectorLength() {
        return sqrt(((x * x) + (y * y)));
    }

    public Point2D invert() {
        x = -x;
        y = -y;
        return this;
    }

    public void roundToInteger() {
        x = (int) x;
        y = (int) y;
    }

    public Point2D subtract(final Point2D point) {
        x -= point.x;
        y -= point.y;
        return this;
    }

    public Point3D to3D() {
        return new Point3D(x, y, 0);
    }

    public Point2D zero() {
        x = 0;
        y = 0;
        return this;
    }

}
